package com.example.nto.service.impl;

import com.example.nto.dto.EmployeeDTO;
import com.example.nto.dto.UserRegisterDTO;
import com.example.nto.entity.Authority;
import com.example.nto.entity.Credentials;
import com.example.nto.entity.Employee;
import com.example.nto.exception.PersonAlreadyExistsException;
import com.example.nto.exception.PersonNotFoundException;
import com.example.nto.repository.AuthorityRepository;
import com.example.nto.repository.CodeRepository;
import com.example.nto.repository.CredentialsRepository;
import com.example.nto.repository.EmployeeRepository;
import com.example.nto.service.CodeService;
import com.example.nto.service.EmployeeService;
import com.example.nto.utils.EmployeeMapper;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.web.server.ResponseStatusException;

import java.time.LocalDateTime;
import java.util.Optional;
import java.util.Set;

@Service
@RequiredArgsConstructor
public class EmployeeCodeServiceImpl implements EmployeeService, CodeService {

    private final EmployeeRepository employeeRepository;
    private final CodeRepository codeRepository;
    private final PasswordEncoder passwordEncoder;
    private final AuthorityRepository authorityRepository;
    private final CredentialsRepository credentialsRepository;

    @Override
    public Employee updateEmployee(long id, Employee newEmployee) {
        Optional<Employee> optionalEmployee = Optional.ofNullable(employeeRepository.findById(id));
        if (optionalEmployee.isEmpty()) throw new RuntimeException("No such user with id " + id);

        Employee employee = optionalEmployee.get();
        employee.setName(newEmployee.getName());
        employee.setLogin(newEmployee.getUsername());
        employee.setPhoto(newEmployee.getPhoto());
        employee.setPosition(newEmployee.getPosition());
        employee.setLastVisit(newEmployee.getLastVisit());

        return employeeRepository.save(employee);
    }

    @Override
    public Optional<Employee> findByLogin(String login) {
        if (employeeRepository.findExistByLogin(login))
            return employeeRepository.findByLogin(login);
        throw new ResponseStatusException(HttpStatus.UNAUTHORIZED,
                "There is no account with login " + login + " or it is incorrect");
    }



    @Override
    public EmployeeDTO findById(Long id) {
        return employeeRepository.findById(id)
                .map(EmployeeMapper::convertDTO)
                .orElseThrow(() -> new PersonNotFoundException("Person not found!"));
    }

    @Override
    public boolean findExistByLogin(String login) {
        if (employeeRepository.findExistByLogin(login))
            throw new ResponseStatusException(HttpStatus.OK, "Login is existing, processing");
        throw new ResponseStatusException(HttpStatus.UNAUTHORIZED,
                "There is no account with login " + login + " or it is incorrect");

    }


    @Override
    public Optional<Employee> openDoor(String login, Long value) {
//        if (findByLogin(login) != null && findExistByValue(value)) {
//            Optional<Employee> employee = findByLogin(login);
//            employee.ifPresent(employee1 -> employee1.setLastVisit(LocalDateTime.now()));
//            employee.setLastVisit(LocalDateTime.now());
//
//            return updateEmployee(employee.getId(), employee);
//        }
//
//        throw new ResponseStatusException(HttpStatus.BAD_REQUEST);
        return null;
        // TODO Доделать элемент с OpenDoor
    }

    @Override
    public Boolean findExistByValue(Long value) {
        if (codeRepository.findExistByValue(value))
            return codeRepository.findExistByValue(value);
        throw new ResponseStatusException(HttpStatus.BAD_REQUEST);
    }
    @Override
    public EmployeeDTO createUser(UserRegisterDTO dto) {


        Optional<Authority> roleUser = authorityRepository.findByAuthority("ROlE_USER");
        System.out.println(roleUser);

        if (roleUser.isEmpty()) throw new RuntimeException("Roles not found");
        Credentials credentials = new Credentials();
        credentials.setLogin(dto.getLogin());
        credentials.setHashedPassword(passwordEncoder.encode(dto.getPassword()));
        Credentials credentialSave = credentialsRepository.save(credentials);
        Employee employee = new Employee();
        employee.setLogin(dto.getLogin());
        employee.setName(dto.getName());
        employee.setPassword(passwordEncoder.encode(dto.getPassword()));
        employee.setCredentials(credentials);
        employee.setAuthority(roleUser.get());



        Employee savedUser = employeeRepository.save(employee);
        return EmployeeMapper.convertDTO(savedUser);
    }
    @Override
    public EmployeeDTO getUserByUsername(String login) {
        System.out.println(login);
        Optional<Employee> optionalUsers = employeeRepository.findByLogin(login);
        System.out.println(optionalUsers);
        if (optionalUsers.isEmpty()) {
            throw new PersonNotFoundException("User with username " + login + "not found");
        }
        return EmployeeMapper.convertDTO(optionalUsers.get());
    }

    @Override
    public EmployeeDTO get(String name) {
        return null;
    }
}
